@@ -1,4 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
+ * Copyright 2011-2022 Pierre Ossman for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +44,13 @@
 #ifndef __RFB_CONFIGURATION_H__
 #define __RFB_CONFIGURATION_H__
 
-#include <rfb/util.h>
+#include <limits.h>
+#include <stdint.h>
+
+#include <string>
+#include <vector>
+
+namespace os { class Mutex; }
 
 namespace rfb {
   class VoidParameter;
@@ -57,15 +64,10 @@ namespace rfb {
   class Configuration {
   public:
     // - Create a new Configuration object
-    Configuration(const char* name, Configuration* attachToGroup=0);
+    Configuration(const char* name_) : name(name_), head(0), _next(0) {}
 
     // - Return the buffer containing the Configuration's name
-    const char* getName() const { return name.buf; }
-
-    // - Assignment operator.  For every Parameter in this Configuration's
-    //   group, get()s the corresponding source parameter and copies its
-    //   content.
-    Configuration& operator=(const Configuration& src);
+    const char* getName() const { return name.c_str(); }
 
     // - Set named parameter to value
     bool set(const char* param, const char* value, bool immutable=false);
@@ -83,6 +85,9 @@ namespace rfb {
     // - List the parameters of this Configuration group
     void list(int width=79, int nameWidth=10);
 
+    // - Remove a parameter from this Configuration group
+    bool remove(const char* param);
+
     // - readFromFile
     //   Read configuration parameters from the specified file.
     void readFromFile(const char* filename);
@@ -119,13 +124,16 @@ namespace rfb {
     static void listParams(int width=79, int nameWidth=10) {
       global()->list(width, nameWidth);
     }
+    static bool removeParam(const char* param) {
+      return global()->remove(param);
+    }
 
   private:
     friend class VoidParameter;
     friend struct ParameterIterator;
 
     // Name for this Configuration
-    CharArray name;
+    std::string name;
 
     // - Pointer to first Parameter in this group
     VoidParameter* head;
@@ -165,13 +173,11 @@ namespace rfb {
 
     virtual bool setParam(const char* value)  = 0;
     virtual bool setParam();
-    virtual char* getDefaultStr() const = 0;
-    virtual char* getValueStr() const = 0;
+    virtual std::string getDefaultStr() const = 0;
+    virtual std::string getValueStr() const = 0;
     virtual bool isBool() const;
 
     virtual void setImmutable();
-    virtual void setHasBeenSet();
-    bool hasBeenSet();
 
   protected:
     friend class Configuration;
@@ -179,9 +185,10 @@ namespace rfb {
 
     VoidParameter* _next;
     bool immutable;
-    bool _hasBeenSet;
     const char* name;
     const char* description;
+
+    os::Mutex* mutex;
   };
 
   class AliasParameter : public VoidParameter {
@@ -190,8 +197,8 @@ namespace rfb {
 		   ConfigurationObject co=ConfGlobal);
     virtual bool setParam(const char* value);
     virtual bool setParam();
-    virtual char* getDefaultStr() const;
-    virtual char* getValueStr() const;
+    virtual std::string getDefaultStr() const;
+    virtual std::string getValueStr() const;
     virtual bool isBool() const;
     virtual void setImmutable();
   private:
@@ -205,8 +212,8 @@ namespace rfb {
     virtual bool setParam(const char* value);
     virtual bool setParam();
     virtual void setParam(bool b);
-    virtual char* getDefaultStr() const;
-    virtual char* getValueStr() const;
+    virtual std::string getDefaultStr() const;
+    virtual std::string getValueStr() const;
     virtual bool isBool() const;
     operator bool() const;
   protected:
@@ -219,10 +226,11 @@ namespace rfb {
     IntParameter(const char* name_, const char* desc_, int v,
                  int minValue=INT_MIN, int maxValue=INT_MAX,
 		 ConfigurationObject co=ConfGlobal);
+    using VoidParameter::setParam;
     virtual bool setParam(const char* value);
     virtual bool setParam(int v);
-    virtual char* getDefaultStr() const;
-    virtual char* getValueStr() const;
+    virtual std::string getDefaultStr() const;
+    virtual std::string getValueStr() const;
     operator int() const;
   protected:
     int value;
@@ -238,47 +246,42 @@ namespace rfb {
 		    ConfigurationObject co=ConfGlobal);
     virtual ~StringParameter();
     virtual bool setParam(const char* value);
-    virtual char* getDefaultStr() const;
-    virtual char* getValueStr() const;
-    void setDefaultStr(const char* v);
+    virtual std::string getDefaultStr() const;
+    virtual std::string getValueStr() const;
     operator const char*() const;
-
-    // getData() returns a copy of the data - it must be delete[]d by the
-    // caller.
-    char* getData() const { return getValueStr(); }
   protected:
-    char* value;
-    const char* def_value;
+    std::string value;
+    std::string def_value;
   };
 
   class BinaryParameter : public VoidParameter {
   public:
-    BinaryParameter(const char* name_, const char* desc_, const void* v, int l,
-		    ConfigurationObject co=ConfGlobal);
+    BinaryParameter(const char* name_, const char* desc_,
+                    const uint8_t* v, size_t l,
+                    ConfigurationObject co=ConfGlobal);
+    using VoidParameter::setParam;
     virtual ~BinaryParameter();
     virtual bool setParam(const char* value);
-    virtual void setParam(const void* v, int l);
-    virtual char* getDefaultStr() const;
-    virtual char* getValueStr() const;
+    virtual void setParam(const uint8_t* v, size_t l);
+    virtual std::string getDefaultStr() const;
+    virtual std::string getValueStr() const;
 
-    // getData() will return length zero if there is no data
-    // NB: data may be set to zero, OR set to a zero-length buffer
-    void getData(void** data, int* length) const;
+    std::vector<uint8_t> getData() const;
 
   protected:
-    char* value;
-    int length;
-    char* def_value;
-    int def_length;
+    uint8_t* value;
+    size_t length;
+    uint8_t* def_value;
+    size_t def_length;
   };
 
   // -=- ParameterIterator
-  //     Iterates over all the Parameters in a Configuration group.  The
-  //     current Parameter is accessed via param, the current Configuration
-  //     via config.  The next() method moves on to the next Parameter.
+  //     Iterates over all enabled parameters (global + server/viewer).
+  //     Current Parameter is accessed via param, the current Configuration
+  //     via config. The next() method moves on to the next Parameter.
 
   struct ParameterIterator {
-    ParameterIterator(Configuration* c) : config(c), param(c ? c->head : 0) {}
+    ParameterIterator() : config(Configuration::global()), param(config->head) {}
     void next() {
       param = param->_next;
       while (!param) {
