@@ -18,6 +18,10 @@
 
 // -=- Registry.cxx
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <rfb_win32/Registry.h>
 #include <rfb_win32/Security.h>
 #include <rdr/MemOutStream.h>
@@ -25,6 +29,7 @@
 #include <rdr/HexInStream.h>
 #include <stdlib.h>
 #include <rfb/LogWriter.h>
+#include <rfb/util.h>
 
 // These flags are required to control access control inheritance,
 // but are not defined by VC6's headers.  These definitions comes
@@ -44,9 +49,9 @@ using namespace rfb::win32;
 static LogWriter vlog("Registry");
 
 
-RegKey::RegKey() : key(0), freeKey(false), valueNameBufLen(0) {}
+RegKey::RegKey() : key(0), freeKey(false), valueName(NULL), valueNameBufLen(0) {}
 
-RegKey::RegKey(const HKEY k) : key(0), freeKey(false), valueNameBufLen(0) {
+RegKey::RegKey(const HKEY k) : key(0), freeKey(false), valueName(NULL), valueNameBufLen(0) {
   LONG result = RegOpenKeyEx(k, 0, 0, KEY_ALL_ACCESS, &key);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegOpenKeyEx(HKEY)", result);
@@ -54,7 +59,7 @@ RegKey::RegKey(const HKEY k) : key(0), freeKey(false), valueNameBufLen(0) {
   freeKey = true;
 }
 
-RegKey::RegKey(const RegKey& k) : key(0), freeKey(false), valueNameBufLen(0) {
+RegKey::RegKey(const RegKey& k) : key(0), freeKey(false), valueName(NULL), valueNameBufLen(0) {
   LONG result = RegOpenKeyEx(k.key, 0, 0, KEY_ALL_ACCESS, &key);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegOpenKeyEx(RegKey&)", result);
@@ -64,6 +69,7 @@ RegKey::RegKey(const RegKey& k) : key(0), freeKey(false), valueNameBufLen(0) {
 
 RegKey::~RegKey() {
   close();
+  delete [] valueName;
 }
 
 
@@ -75,24 +81,24 @@ void RegKey::setHKEY(HKEY k, bool fK) {
 }
 
 
-bool RegKey::createKey(const RegKey& root, const TCHAR* name) {
+bool RegKey::createKey(const RegKey& root, const char* name) {
   close();
   LONG result = RegCreateKey(root.key, name, &key);
   if (result != ERROR_SUCCESS) {
     vlog.error("RegCreateKey(%p, %s): %lx", root.key, name, result);
     throw rdr::SystemException("RegCreateKeyEx", result);
   }
-  vlog.debug("createKey(%p,%s) = %p", root.key, (const char*)CStr(name), key);
+  vlog.debug("createKey(%p,%s) = %p", root.key, name, key);
   freeKey = true;
   return true;
 }
 
-void RegKey::openKey(const RegKey& root, const TCHAR* name, bool readOnly) {
+void RegKey::openKey(const RegKey& root, const char* name, bool readOnly) {
   close();
   LONG result = RegOpenKeyEx(root.key, name, 0, readOnly ? KEY_READ : KEY_ALL_ACCESS, &key);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegOpenKeyEx (open)", result);
-  vlog.debug("openKey(%p,%s,%s) = %p", root.key, (const char*)CStr(name),
+  vlog.debug("openKey(%p,%s,%s) = %p", root.key, name,
 	         readOnly ? "ro" : "rw", key);
   freeKey = true;
 }
@@ -114,13 +120,13 @@ void RegKey::close() {
   }
 }
 
-void RegKey::deleteKey(const TCHAR* name) const {
+void RegKey::deleteKey(const char* name) const {
   LONG result = RegDeleteKey(key, name);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegDeleteKey", result);
 }
 
-void RegKey::deleteValue(const TCHAR* name) const {
+void RegKey::deleteValue(const char* name) const {
   LONG result = RegDeleteValue(key, name);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegDeleteValue", result);
@@ -136,62 +142,60 @@ void RegKey::awaitChange(bool watchSubTree, DWORD filter, HANDLE event) const {
 RegKey::operator HKEY() const {return key;}
 
 
-void RegKey::setExpandString(const TCHAR* valname, const TCHAR* value) const {
-  LONG result = RegSetValueEx(key, valname, 0, REG_EXPAND_SZ, (const BYTE*)value, (_tcslen(value)+1)*sizeof(TCHAR));
+void RegKey::setExpandString(const char* valname, const char* value) const {
+  LONG result = RegSetValueEx(key, valname, 0, REG_EXPAND_SZ, (const BYTE*)value, (strlen(value)+1)*sizeof(char));
   if (result != ERROR_SUCCESS) throw rdr::SystemException("setExpandString", result);
 }
 
-void RegKey::setString(const TCHAR* valname, const TCHAR* value) const {
-  LONG result = RegSetValueEx(key, valname, 0, REG_SZ, (const BYTE*)value, (_tcslen(value)+1)*sizeof(TCHAR));
+void RegKey::setString(const char* valname, const char* value) const {
+  LONG result = RegSetValueEx(key, valname, 0, REG_SZ, (const BYTE*)value, (strlen(value)+1)*sizeof(char));
   if (result != ERROR_SUCCESS) throw rdr::SystemException("setString", result);
 }
 
-void RegKey::setBinary(const TCHAR* valname, const void* value, int length) const {
+void RegKey::setBinary(const char* valname, const void* value, size_t length) const {
   LONG result = RegSetValueEx(key, valname, 0, REG_BINARY, (const BYTE*)value, length);
   if (result != ERROR_SUCCESS) throw rdr::SystemException("setBinary", result);
 }
 
-void RegKey::setInt(const TCHAR* valname, int value) const {
+void RegKey::setInt(const char* valname, int value) const {
   LONG result = RegSetValueEx(key, valname, 0, REG_DWORD, (const BYTE*)&value, sizeof(value));
   if (result != ERROR_SUCCESS) throw rdr::SystemException("setInt", result);
 }
 
-void RegKey::setBool(const TCHAR* valname, bool value) const {
+void RegKey::setBool(const char* valname, bool value) const {
   setInt(valname, value ? 1 : 0);
 }
 
-TCHAR* RegKey::getString(const TCHAR* valname) const {return getRepresentation(valname);}
-TCHAR* RegKey::getString(const TCHAR* valname, const TCHAR* def) const {
+std::string RegKey::getString(const char* valname) const {
+  return getRepresentation(valname);
+}
+
+std::string RegKey::getString(const char* valname, const char* def) const {
   try {
     return getString(valname);
   } catch(rdr::Exception&) {
-    return tstrDup(def);
+    return def;
   }
 }
 
-void RegKey::getBinary(const TCHAR* valname, void** data, int* length) const {
-  TCharArray hex(getRepresentation(valname));
-  if (!rdr::HexInStream::hexStrToBin(CStr(hex.buf), (char**)data, length))
-    throw rdr::Exception("getBinary failed");
+std::vector<uint8_t> RegKey::getBinary(const char* valname) const {
+  std::string hex = getRepresentation(valname);
+  return hexToBin(hex.data(), hex.size());
 }
-void RegKey::getBinary(const TCHAR* valname, void** data, int* length, void* def, int deflen) const {
+std::vector<uint8_t> RegKey::getBinary(const char* valname, const uint8_t* def, size_t deflen) const {
   try {
-    getBinary(valname, data, length);
+    return getBinary(valname);
   } catch(rdr::Exception&) {
-    if (deflen) {
-      *data = new char[deflen];
-      memcpy(*data, def, deflen);
-    } else
-      *data = 0;
-    *length = deflen;
+    std::vector<uint8_t> out(deflen);
+    memcpy(out.data(), def, deflen);
+    return out;
   }
 }
 
-int RegKey::getInt(const TCHAR* valname) const {
-  TCharArray tmp(getRepresentation(valname));
-  return _ttoi(tmp.buf);
+int RegKey::getInt(const char* valname) const {
+  return atoi(getRepresentation(valname).c_str());
 }
-int RegKey::getInt(const TCHAR* valname, int def) const {
+int RegKey::getInt(const char* valname, int def) const {
   try {
     return getInt(valname);
   } catch(rdr::Exception&) {
@@ -199,65 +203,54 @@ int RegKey::getInt(const TCHAR* valname, int def) const {
   }
 }
 
-bool RegKey::getBool(const TCHAR* valname) const {
+bool RegKey::getBool(const char* valname) const {
   return getInt(valname) > 0;
 }
-bool RegKey::getBool(const TCHAR* valname, bool def) const {
+bool RegKey::getBool(const char* valname, bool def) const {
   return getInt(valname, def ? 1 : 0) > 0;
 }
 
-static inline TCHAR* terminateData(char* data, int length)
-{
-  // We must terminate the string, just to be sure.  Stupid Win32...
-  int len = length/sizeof(TCHAR);
-  TCharArray str(len+1);
-  memcpy(str.buf, data, length);
-  str.buf[len] = 0;
-  return str.takeBuf();
-}
-
-TCHAR* RegKey::getRepresentation(const TCHAR* valname) const {
+std::string RegKey::getRepresentation(const char* valname) const {
   DWORD type, length;
   LONG result = RegQueryValueEx(key, valname, 0, &type, 0, &length);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("get registry value length", result);
-  CharArray data(length);
-  result = RegQueryValueEx(key, valname, 0, &type, (BYTE*)data.buf, &length);
+  std::vector<uint8_t> data(length);
+  result = RegQueryValueEx(key, valname, 0, &type, (BYTE*)data.data(), &length);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("get registry value", result);
 
   switch (type) {
   case REG_BINARY:
     {
-      TCharArray hex(rdr::HexOutStream::binToHexStr(data.buf, length));
-      return hex.takeBuf();
+      return binToHex(data.data(), length);
     }
   case REG_SZ:
     if (length) {
-      return terminateData(data.buf, length);
+      return std::string((char*)data.data(), length);
     } else {
-      return tstrDup(_T(""));
+      return "";
     }
   case REG_DWORD:
     {
-      TCharArray tmp(16);
-      _stprintf(tmp.buf, _T("%lu"), *((DWORD*)data.buf));
-      return tmp.takeBuf();
+      char tmp[16];
+      sprintf(tmp, "%lu", *((DWORD*)data.data()));
+      return tmp;
     }
   case REG_EXPAND_SZ:
     {
     if (length) {
-      TCharArray str(terminateData(data.buf, length));
-      DWORD required = ExpandEnvironmentStrings(str.buf, 0, 0);
+      std::string str((char*)data.data(), length);
+      DWORD required = ExpandEnvironmentStrings(str.c_str(), 0, 0);
       if (required==0)
         throw rdr::SystemException("ExpandEnvironmentStrings", GetLastError());
-      TCharArray result(required);
-      length = ExpandEnvironmentStrings(str.buf, result.buf, required);
+      std::vector<char> result(required);
+      length = ExpandEnvironmentStrings(str.c_str(), result.data(), required);
       if (required<length)
         throw rdr::Exception("unable to expand environment strings");
-      return result.takeBuf();
+      return result.data();
     } else {
-      return tstrDup(_T(""));
+      return "";
     }
     }
   default:
@@ -265,47 +258,47 @@ TCHAR* RegKey::getRepresentation(const TCHAR* valname) const {
   }
 }
 
-bool RegKey::isValue(const TCHAR* valname) const {
+bool RegKey::isValue(const char* valname) const {
   try {
-    TCharArray tmp(getRepresentation(valname));
+    getRepresentation(valname);
     return true;
   } catch(rdr::Exception&) {
     return false;
   }
 }
 
-const TCHAR* RegKey::getValueName(int i) {
+const char* RegKey::getValueName(int i) {
   DWORD maxValueNameLen;
   LONG result = RegQueryInfoKey(key, 0, 0, 0, 0, 0, 0, 0, &maxValueNameLen, 0, 0, 0);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegQueryInfoKey", result);
   if (valueNameBufLen < maxValueNameLen + 1) {
     valueNameBufLen = maxValueNameLen + 1;
-    delete [] valueName.buf;
-    valueName.buf = new TCHAR[valueNameBufLen];
+    delete [] valueName;
+    valueName = new char[valueNameBufLen];
   }
   DWORD length = valueNameBufLen;
-  result = RegEnumValue(key, i, valueName.buf, &length, NULL, 0, 0, 0);
+  result = RegEnumValue(key, i, valueName, &length, NULL, 0, 0, 0);
   if (result == ERROR_NO_MORE_ITEMS) return 0;
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegEnumValue", result);
-  return valueName.buf;
+  return valueName;
 }
 
-const TCHAR* RegKey::getKeyName(int i) {
+const char* RegKey::getKeyName(int i) {
   DWORD maxValueNameLen;
   LONG result = RegQueryInfoKey(key, 0, 0, 0, 0, &maxValueNameLen, 0, 0, 0, 0, 0, 0);
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegQueryInfoKey", result);
   if (valueNameBufLen < maxValueNameLen + 1) {
     valueNameBufLen = maxValueNameLen + 1;
-    delete [] valueName.buf;
-    valueName.buf = new TCHAR[valueNameBufLen];
+    delete [] valueName;
+    valueName = new char[valueNameBufLen];
   }
   DWORD length = valueNameBufLen;
-  result = RegEnumKeyEx(key, i, valueName.buf, &length, NULL, 0, 0, 0);
+  result = RegEnumKeyEx(key, i, valueName, &length, NULL, 0, 0, 0);
   if (result == ERROR_NO_MORE_ITEMS) return 0;
   if (result != ERROR_SUCCESS)
     throw rdr::SystemException("RegEnumKey", result);
-  return valueName.buf;
+  return valueName;
 }
