@@ -18,6 +18,10 @@
 
 // -=- VNC Server 4.0 for Windows (WinVNC4)
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 
 #include <winvnc/VNCServerWin32.h>
@@ -27,6 +31,7 @@
 #include <rfb/Logger_stdio.h>
 #include <rfb/Logger_file.h>
 #include <rfb/LogWriter.h>
+#include <rfb/util.h>
 #include <rfb_win32/AboutDialog.h>
 #include <rfb_win32/MsgBox.h>
 #include <network/TcpSocket.h>
@@ -37,7 +42,7 @@ using namespace win32;
 
 static LogWriter vlog("main");
 
-TStr rfb::win32::AppName("VNC Server");
+const char* rfb::win32::AppName = "TigerVNC Server";
 
 
 extern bool runAsService;
@@ -52,12 +57,12 @@ static bool close_console = false;
 
 static void programInfo() {
   win32::FileVersionInfo inf;
-  _tprintf(_T("%s - %s, Version %s\n"),
-    inf.getVerString(_T("ProductName")),
-    inf.getVerString(_T("FileDescription")),
-    inf.getVerString(_T("FileVersion")));
+  printf("%s - %s, Version %s\n",
+    inf.getVerString("ProductName"),
+    inf.getVerString("FileDescription"),
+    inf.getVerString("FileVersion"));
   printf("%s\n", buildTime);
-  _tprintf(_T("%s\n\n"), inf.getVerString(_T("LegalCopyright")));
+  printf("%s\n\n", inf.getVerString("LegalCopyright"));
 }
 
 static void programUsage() {
@@ -78,16 +83,16 @@ static void programUsage() {
   printf("\nLog destinations:\n");
   Logger::listLoggers();
   printf("\nAvailable configuration parameters:\n");
-  Configuration::listParams(ConfServer);
+  Configuration::listParams(79, 14);
 }
 
 static void MsgBoxOrLog(const char* msg, bool isError=false) {
   if (close_console) {
-    MsgBox(0, TStr(msg), (isError ? MB_ICONERROR : MB_ICONINFORMATION) | MB_OK);
+    MsgBox(0, msg, (isError ? MB_ICONERROR : MB_ICONINFORMATION) | MB_OK);
   } else {
     if (isError) {
       try {
-        vlog.error(msg);
+        vlog.error("%s", msg);
         return;
       } catch (...) {
       }
@@ -102,30 +107,30 @@ static void processParams(int argc, char** argv) {
 
       if (strcasecmp(argv[i], "-connect") == 0) {
         runServer = false;
-        CharArray host;
+        const char *host = NULL;
         if (i+1 < argc) {
-          host.buf = strDup(argv[i+1]);
+          host = argv[i+1];
           i++;
         } else {
           AddNewClientDialog ancd;
           if (ancd.showDialog())
-            host.buf = strDup(ancd.getHostName());
+            host = ancd.getHostName();
         }
-        if (host.buf) {
-          HWND hwnd = FindWindow(0, _T("winvnc::IPC_Interface"));
+        if (host != NULL) {
+          HWND hwnd = FindWindow(0, "winvnc::IPC_Interface");
           if (!hwnd)
             throw rdr::Exception("Unable to locate existing VNC Server.");
           COPYDATASTRUCT copyData;
           copyData.dwData = 1; // *** AddNewClient
-          copyData.cbData = strlen(host.buf);
-          copyData.lpData = (void*)host.buf;
+          copyData.cbData = strlen(host);
+          copyData.lpData = (void*)host;
           printf("Sending connect request to VNC Server...\n");
           if (!SendMessage(hwnd, WM_COPYDATA, 0, (LPARAM)&copyData))
             MsgBoxOrLog("Connection failed.", true);
         }
       } else if (strcasecmp(argv[i], "-disconnect") == 0) {
         runServer = false;
-        HWND hwnd = FindWindow(0, _T("winvnc::IPC_Interface"));
+        HWND hwnd = FindWindow(0, "winvnc::IPC_Interface");
         if (!hwnd)
           throw rdr::Exception("Unable to locate existing VNC Server.");
         COPYDATASTRUCT copyData;
@@ -148,12 +153,12 @@ static void processParams(int argc, char** argv) {
       } else if (strcasecmp(argv[i], "-status") == 0) {
         printf("Querying service status...\n");
         runServer = false;
+        std::string result;
         DWORD state = rfb::win32::getServiceState(VNCServerService::Name);
-        CharArray stateStr(rfb::win32::serviceStateName(state));
-        const char* stateMsg = "The %s Service is in the %s state.";
-        CharArray result(strlen(stateStr.buf) + _tcslen(VNCServerService::Name) + strlen(stateMsg) + 1);
-        sprintf(result.buf, stateMsg, (const char*)CStr(VNCServerService::Name), stateStr.buf);
-        MsgBoxOrLog(result.buf);
+        result = format("The %s Service is in the %s state.",
+                        VNCServerService::Name,
+                        rfb::win32::serviceStateName(state));
+        MsgBoxOrLog(result.c_str());
       } else if (strcasecmp(argv[i], "-service") == 0) {
         printf("Run in service mode\n");
         runServer = false;
@@ -168,8 +173,22 @@ static void processParams(int argc, char** argv) {
         runServer = false;
         int j = i;
         i = argc;
+
+        // Try to clean up earlier services we've had
+        try {
+          rfb::win32::unregisterService("WinVNC4");
+        } catch (rdr::SystemException&) {
+          // Do nothing as we might fail simply because there was no
+          // service to remove
+        }
+        try {
+          rfb::win32::unregisterService("TigerVNC Server");
+        } catch (rdr::SystemException&) {
+        }
+
         if (rfb::win32::registerService(VNCServerService::Name,
-                                        _T("VNC Server Version 4"),
+                                        "TigerVNC Server",
+                                        "Provides remote access to this machine via the VNC/RFB protocol.",
                                         argc-(j+1), &argv[j+1]))
           MsgBoxOrLog("Registered service successfully");
       } else if (strcasecmp(argv[i], "-unregister") == 0) {
@@ -182,7 +201,7 @@ static void processParams(int argc, char** argv) {
         close_console = true;
         vlog.info("closing console");
         if (!FreeConsole())
-          vlog.info("unable to close console:%u", GetLastError());
+          vlog.info("unable to close console:%lu", GetLastError());
 
       } else if ((strcasecmp(argv[i], "-help") == 0) ||
         (strcasecmp(argv[i], "--help") == 0) ||
@@ -220,7 +239,7 @@ static void processParams(int argc, char** argv) {
 // -=- main
 //
 
-int WINAPI WinMain(HINSTANCE inst, HINSTANCE prevInst, char* cmdLine, int cmdShow) {
+int WINAPI WinMain(HINSTANCE /*inst*/, HINSTANCE /*prevInst*/, char* /*cmdLine*/, int /*cmdShow*/) {
   int result = 0;
 
   try {
